<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Helper\Catalog;

use Magento\Catalog\Api\Data\CategoryInterface;
use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Store\Model\ScopeInterface;

/**
 *
 */
class CategoryHelper
{
    /** @var string */
    public const XML_REFILO_SEARCH_GENERAL_USE_CATEGORY_FULL_PATH = 'refilo_search/general/use_category_full_path';

    /** @var string */
    public const XML_REFILO_SEARCH_GENERAL_CATEGORY_FULL_PATH_SEPARATOR = 'refilo_search/general/category_full_path_separator';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var array
     */
    private $categoryCache = [];
    /**
     * @var CollectionFactory
     */
    private $categoryCollectionFactory;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param CollectionFactory $categoryCollectionFactory
     */
    public function __construct(ScopeConfigInterface $scopeConfig, CollectionFactory $categoryCollectionFactory)
    {
        $this->scopeConfig = $scopeConfig;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
    }

    /**
     * Used to show full name in the search result
     * @param CategoryInterface| Category $category
     */
    public function getFullName($category): ?string
    {
        $useFullName = $this->scopeConfig->isSetFlag
        (
            self::XML_REFILO_SEARCH_GENERAL_USE_CATEGORY_FULL_PATH,
            ScopeInterface::SCOPE_STORE,
            $category->getStore()
        );

        if (!$useFullName) {
            return $category->getName();
        }

        $names = [$category->getName()];
        $parent = $category->getParentCategory();

        while (true) {
            if ((int)$parent->getLevel() <= 1 || !$parent->getParentId()) {
                break;
            }

            array_unshift($names, $parent->getName());

            $parent = $parent->getParentCategory();
        }

        $separator = $this->scopeConfig->getValue
            (
                self::XML_REFILO_SEARCH_GENERAL_CATEGORY_FULL_PATH_SEPARATOR,
                ScopeInterface::SCOPE_STORE,
                $category->getStore()
            ) ?? ' / ';
        return implode($separator, $names);

    }

    /**
     * @param CategoryInterface| Category | int $category
     * @param $store
     * @return void
     * @throws LocalizedException
     */
    public function getBreadcrumbs($category, $store): array
    {
        if (!isset($this->categoryCache[$store][$category])) {

            $items = $this->categoryCollectionFactory->create()
                ->addAttributeToFilter('level', ['gteq' => 2])
                ->addNameToResult()
                ->joinUrlRewrite()
                ->setStore($store)
                ->getItems();

            foreach ($items as $item) {
                $this->categoryCache[$store][$item->getId()] = $item;
            }

        }

        $currentCategory = $this->categoryCache[$store][$category];

        $paths = explode('/', $currentCategory->getPath());

        $result = [];
        foreach ($paths as $path) {

            if (!isset($this->categoryCache[$store][$path])) {
                continue;
            }

            $parentCategory = $this->categoryCache[$store][$path];
            $result[] = [
                'label' => $parentCategory->getName(),
                'url'   => $parentCategory->getRequestPath()
            ];
        }

        return $result;
    }
}
